<?php 
// $Id: content_distribution.views.inc,v 1.1.2.3.2.3 2010/02/08 21:27:50 joachim Exp $
/**
 * @file content_distribution.views.inc
 * Provides support for the Views module.
 *
 */

/**
 * Implementation of hook_views_data_alter()
 * 
 * Replaces the node.changed data for the views module and adds an
 * 'Updated date' argument.
 */
function content_distribution_views_data_alter(&$data) {
  // Add our own argument for node changed date which uses a greater than
  // rather than equality clause.  
  $data['node']['changed_since'] = array(
    'title' => t('Updated since date'), // The item it appears as on the UI,
    'help' => t('Filter nodes updated since the given date.'), // The help that appears on the UI,
    'argument' => array(
      'field' => 'changed',
      'handler' => 'content_distribution_handler_argument_node_updated_since',
    ),
  );  
}

/**
 * Define extra filters for the views module.
 */
function content_distribution_views_data() {
  $data = array();
	
  // Define the base group of this table. Fields that don't
  // have a group defined will go into this field by default.
  $data['content_distribution']['table']['group']  = t('Content Distribution');

  $data['content_distribution']['table']['base'] = array(
    'field' => 'nid',
    'title' => t('Content Distribution'),
    'help' => t("Content distribution relates to nodes which are set up to be distributed remotely."),
  );
  
  $data['content_distribution']['table']['join'] = array(
    'node' => array(
      'left_field' => 'nid',
      'field' => 'nid',
    ),
  );	
  // ----------------------------------------------------------------
  // Fields
  
  //published date
  $data['content_distribution']['published_date'] = array(
    'title' => t('Published Date'),
    'help' => t('Date and time of when node is set to be published.'),
    'field' => array(
      'handler' => 'views_handler_field_date',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_date',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_date',
    ),
  );
  
  //week argument
  $data['content_distribution']['published_week'] = array(
    'title' => t('Published week'),
    'help' => t('In the form of WW (01 - 53).'),
    'argument' => array(
      'field' => 'published_date',
      'handler' => 'views_handler_argument_node_created_week',
    ),
  );
  return $data;
}

/**
 * Implementation of hook_views_handlers().
 */
function content_distribution_views_handlers() {
  return array(
    'info' => array(
      'path' => drupal_get_path('module', 'content_distribution') . '/includes',
    ),
    'handlers' => array(
      'content_distribution_handler_argument_node_updated_since' => array(
        'parent' => 'views_handler_argument_date',
      ),
    ),
  );
}
