<?php
// $Id: content_distribution_nodequeue.content_retriever_arguments_process.inc,v 1.1.2.1 2010/02/11 13:14:35 joachim Exp $

/**
 * @file
 * Contains our implementation of hook_content_retriever_views_arguments_process.
 */

/**
 * Implementation of hook_content_retriever_views_arguments_process.
 *
 * Make nicer form widgets for the remote view arguments form.
 *
 * Modules implementing this hook should iterate over $view_arguments, and for
 * argument types they know about, insert a form element into the $results array
 * at the same key.
 *
 * @param $view_arguments
 *  The arguments defined on the remote view.
 *
 * @param $results
 *  An array into which to put form elements. This is prefilled with the same
 *  keys as $view_arguments. Modules implementing this hook should place any
 *  form elements they wish to contribute into this array, at the right key.
 *  If an item is not NULL (ie, a prior implementing module has already added
 *  data) then it should not be replaced.
 *  The #default_value key will be set by the content_retriever module.
 */
function content_distribution_nodequeue_content_retriever_views_arguments_process($view_arguments, &$results) {
  foreach ($view_arguments as $argument_id => $argument) {
    // Do not consider any arguments that have already had their result filled in.
    if (!is_null($results[$argument_id])) {
      continue;
    }
    
    $options = array();

    // ----------------------------------------------------------------
    // Nodequeue argument
    if ($argument['table'] == 'nodequeue_nodes' && $argument['field'] == 'qid') {
      $queues = content_retriever_xmlrpc('nodequeue.getQueues');

      if ($queues) {
        foreach ($queues as $queue) {
          $options[$queue['qid']] = $queue['title'];
        }
        
        $results[$argument_id] = array(
          '#title' => t('Nodequeues to interrogate'),
          '#type' => 'checkboxes',
          '#options' => $options,
        );
      } 
      else {
        drupal_set_message(t('No nodequeues available.'));
      }
    }
    
  }
}

