<?php
// $Id: content_distribution_nodereference.content_retriever_arguments_process.inc,v 1.1.2.1 2010/02/11 11:36:05 joachim Exp $

/**
 * @file
 * Contains our implementation of hook_content_retriever_views_arguments_process.
 */

/**
 * Implementation of hook_content_retriever_views_arguments_process.
 *
 * Make nicer form widgets for the remote view arguments form.
 *
 * Modules implementing this hook should iterate over $view_arguments, and for
 * argument types they know about, insert a form element into the $results array
 * at the same key.
 *
 * @param $view_arguments
 *  The arguments defined on the remote view.
 *
 * @param $results
 *  An array into which to put form elements. This is prefilled with the same
 *  keys as $view_arguments. Modules implementing this hook should place any
 *  form elements they wish to contribute into this array, at the right key.
 *  If an item is not NULL (ie, a prior implementing module has already added
 *  data) then it should not be replaced.
 *  The #default_value key will be set by the content_retriever module.
 */
function content_distribution_nodereference_content_retriever_views_arguments_process($view_arguments, &$results) {
  foreach ($view_arguments as $argument_id => $argument) {
    // Do not consider any arguments that have already had their result filled in.
    if (!is_null($results[$argument_id])) {
      continue;
    }
    
    $options = array();
    
    // ----------------------------------------------------------------
    // CCK nodereference argument
    // CCK has lots of names for tables and fields, but they all have 
    // standard prefixes:
    // - table is node_data_FOO
    // - field is field_FOO
    // - nodereference field is field_FOO_nid
    if (substr($argument['table'], 0, 10)  == 'node_data_' 
        && substr($argument['field'], 0, 6) == 'field_' 
        && substr($argument['field'], -4) == '_nid') {
      // Get the fieldname and thence the field
      $fieldname = substr($argument['field'], 0, -4);
      $field = content_retriever_xmlrpc('content.contentFields', $fieldname);
      
      if ($field['type'] == 'nodereference') {
        $nodes = content_retriever_xmlrpc('content.nodereferencePotentialReferences', $field);
        
        foreach ($nodes as $nid => $data) {
          $options[$nid] = $data['title'];
        }

        $results[$argument_id] = array(
  	      '#title' => t('Node reference field %fieldname', array('%fieldname' => $fieldname)),
  	      '#type' => $field['multiple'] ? 'checkboxes' : 'radios',
  	      '#options' => $options,
  	    );	  
      }
    }
  }
}

