<?php
// $Id: content_retriever.api.php,v 1.1.2.1 2010/02/10 22:31:08 joachim Exp $


/**
 * @file
 * This file documents the hooks invoked by the Content retriever module.
 */

/**
 * Hook to make nicer form widgets for the remote view arguments form.
 *
 * This should be implemented in file MODULENAME.content_retriever_arguments_process.inc
 *
 * Modules implementing this hook should iterate over $view_arguments, and for
 * argument types they know about, insert a form element into the $results array
 * at the same key.
 *
 * @param $view_arguments
 *  The arguments defined on the remote view.
 *
 * @param $results
 *  An array into which to put form elements. This is prefilled with the same
 *  keys as $view_arguments. Modules implementing this hook should place any
 *  form elements they wish to contribute into this array, at the right key.
 *  If an item is not NULL (ie, a prior implementing module has already added
 *  data) then it should not be replaced.
 *  The #default_value key will be set by the content_retriever module.
 */
function hook_content_retriever_views_arguments_process($view_arguments, &$results) {
  foreach ($view_arguments as $argument_id => $argument) {
    // Do not consider any arguments that have already had their result filled in.
    if (!is_null($results[$argument_id])) {
      continue;
    }
    
    $options = array();
    
    if ($argument['table'] == 'MINE' && $argument['field'] == 'ALSO MINE') {
      $results[$argument_id] = array(
	      '#title' => t('MY ARGUMENT'),
	      '#type' => 'radios',
	      '#options' => $options,
	    );	  
    }
  }
}

/**
 * Hook to alter retrieved nodes before they are acted on by other modules.
 *
 * This should be implemented in file MODULENAME.content_retriever_node_process.inc
 *
 * Modules may alter the $node object here before hook_content_retriever_node_presave()
 * is called.
 */
function hook_content_retriever_node_alter(&$node, $node_exists_locally) {
  unset($node->MYPROPERTY);
}

/**
 * Hook to act on retrieved nodes before they are saved.
 *
 * This should be implemented in file MODULENAME.content_retriever_node_process.inc
 *
 * Modules should implement this hook if they need to act on the $node object
 * to save data before the node_save() call.
 * For instance, modules may need to add properties to the $node object so 
 * their hook_nodeapi() save operation correctly saves data.
 * For an example, see content_retriever_process_filefield module.
 */
function hook_content_retriever_node_presave($node, $node_exists_locally) {

}

/**
 * Hook to act on retrieved nodes after they are saved.
 *
 * This should be implemented in file MODULENAME.content_retriever_node_process.inc
 *
 * Modules should implement this hook if they need to act on the $node object
 * to save data after the node_save() call; in particular if they need a nid
 * in order to save data to their own tables.
 */
function hook_content_retriever_node_postsave($node, $node_exists_locally) {

}
