<?php
// $Id: content_retriever.content_retriever_node_process.inc,v 1.1.2.1 2010/02/10 21:51:39 joachim Exp $

/**
 * @file
 * Contains our implementations of content_retriever node hooks:
 *  - hook_content_retriever_node_alter: alter a node before it is saved.
 *  - hook_content_retriever_node_presave: act on data from an incoming node 
 *      before it is saved.
 *  - hook_content_retriever_node_postsave: act on data from an incoming node
 *      after it is saved
 */

/**
 * Implement hook_content_retriever_node_alter on behalf of path module.
 */
function path_content_retriever_node_alter(&$node, $node_exists_locally) {
  unset($node->path);
}

/**
 * Implement hook_content_retriever_node_alter on behalf of comment module.
 */
function comment_content_retriever_node_alter(&$node, $node_exists_locally) {
  unset($node->comment_count);
  unset($node->last_comment_timestamp);
  unset($node->last_comment_name);
}

/**
 * Implement hook_content_retriever_node_alter on behalf of nodequeue module.
 */
function nodequeue_content_retriever_node_alter(&$node, $node_exists_locally) {
  unset($node->queues);
}

/**
 * Implement hook_content_retriever_node_alter on behalf of taxonomy module.
 */
function taxonomy_content_retriever_node_alter(&$node, $node_exists_locally) {
  // Taxonomy parsing - rewrites the taxonomy object
  if (module_exists('taxonomy_parser')) {
    if ($node->taxonomy) {
      // construct array of term names from the $node->taxonomy
      $term_categories = array();
      foreach ($node->taxonomy as $term) {         
        $term_categories[$term['name']]['vocab'] = $node->vocab_names[$term['tid']];
      }
      // parse the taxonomy and return taxonomy array to be attached to the node object
      $node->taxonomy = taxonomy_parser_import_taxonomy($term_categories, $node->type);   
    }
  }
  else {
  	// Module is not installed so capture the issue
    watchdog('integration', 'Failed to call taxonomy_parser_import_taxonomy(). The taxonomy_parser module is not available.', array(), WATCHDOG_WARNING);
    unset($node->taxonomy);
  }
}



