<?php
// $Id: content_retriever_process_filefield.content_retriever_node_process.inc,v 1.1.2.3 2010/07/08 17:08:29 joachim Exp $

/**
 * @file content_retriever_process_filefield.content_retriever_node_process.inc
 * 
 * Handles saving of filefield (and hence also imagefield) files and node data.
 */

/**
 * Implementation of content_retriever_node_save().
 *
 * Fetch imagefield / filefield files and store data about them into the $node object.
 */
function content_retriever_process_filefield_content_retriever_node_presave($node, $node_exists_locally) {
  // Get CCK's data about the local node type and thence a list of its fields.
  $content_type_info = content_types($node->type);
	$fields = $content_type_info['fields'];

  $files_array = array();
  
  //check the cck field exists on the local content type
  if (!empty($fields)) {
    foreach ($fields as $content_type_field) {
      if (($content_type_field['module'] == 'imagefield') || ($content_type_field['module'] == 'filefield')) {
	      if (!empty($node->$content_type_field['field_name'])) {
          $files_array[$content_type_field['field_name']] = $node->$content_type_field['field_name'];
      }
      }
    }
  }
  
  $file_count = count($files_array);
  if ($file_count > 0) {
    foreach ($files_array as $fieldname => $file_type) {  
      $field_array = $node->$fieldname;
      foreach ($file_type as $key => $incoming_file) {
        $file = content_retriever_xmlrpc('file.get', (int)$incoming_file['fid']);
        $file_details = _content_retriever_process_filefield_save_file($file, $node->nid, NULL, $node->type, $fields[$fieldname]);
      
        $field_array[$key]['fid'] = $file_details->fid;
        $field_array[$key]['filepath'] = $file_details->filepath;                             
      }
      $node->$fieldname = $field_array;
    }
  }
}

/**
 * Save a file retrieved from the remote server into a CCK filefield.
 *
 * @param $file
 *  The result of the 'file.get' method: an array of file data
 * @param $nid
 *  integer: the NEW node id on the local instance
 * @param $vid
 *  the NEW node revision id on the local instance
 * @param $node_type
 *  The node type.
 * @param $field
 *  The field array this file is in, as returned by CCK's content_fields().
 *
 * @return
 *  An array of file details containing the fid and filepath of the file as
 *  saved into the {files} table.
 */
function _content_retriever_process_filefield_save_file($file, $nid, $vid = null, $node_type = '', $field = NULL) {
  $file_data = base64_decode($file['file']);

  // Get filepath to save to, using the settings from the CCK widget.
  // filefield module's filefield_widget_file_path() gets us a complete
  // path to the right directory.
  $destination = filefield_widget_file_path($field);
  if (!field_file_check_directory($destination, FILE_CREATE_DIRECTORY)) {
    watchdog('filefield', 'The upload directory %directory for the file field %field (content type %type) could not be created or is not accessible. A newly uploaded file could not be saved in this directory as a consequence, and the upload was canceled.', array(
    '%directory' => $destination, 
    '%field' => $field['field_name'], 
    '%type' => $node_type,
  ));
    return NULL;
  }

  $file['filepath'] = filefield_widget_file_path($field) . '/' . $file['filename'];
  
  // Set content type db table, and field to search on.
  // We get the details of the filefield table from CCK
  // Then use the table name and file ID column, this will
  // work for both single and multiple filefield fields
  $db_info = content_database_info($field);
  $content_table = $db_info['table'];
  $fid_field = $db_info['columns']['fid']['column'];
    
  $file_details = array();
  // Get timestamp and fid if there is a file on this node.
  $result = db_query("SELECT f.timestamp, f.fid, f.filepath FROM {files} as f
    RIGHT JOIN {%s} as ctype
    ON f.fid=ctype.%s
    WHERE ctype.vid = (SELECT MAX({%s}.vid)
    FROM {%s}) AND ctype.nid = %d", $content_table, $fid_field, $content_table, $content_table, $nid);

  $file_details = db_fetch_object($result);
  
  // Check if it's the same file coming over.
  if ($file_details->timestamp == $file['timestamp']) {
    // If it is it dont write another record in the files table.
    return $file_details;
  }

  // Save the file.
  // note: safe to REPLACE because we can be certain we will not have duplicates
  // we wish to keep - distribution site will be handling duplicate file names
  // so we will only ever receive unique ones here
  file_save_data($file_data, $file['filepath'], FILE_EXISTS_REPLACE);

  // Write record in files table.
  drupal_write_record('files', $file);

  $result = db_query("SELECT fid, filepath FROM {files} WHERE filename = '%s'", $file['filename']);
  $file_details = db_fetch_object($result);

  return $file_details;
}
