<?php
// $Id: distributor_service.inc,v 1.1.2.4.2.4 2010/02/11 21:31:12 joachim Exp $
/**
 * @author gtaylor
 * @file
 * Define services for content distribution.
 */


/**
 * Returns a specified node.
 *
 * @param $type
 *   String. The node Type.
 * 
 * @return
 *   Boolean. The node is set up to be distributed or not.
 */
function distributor_service_get_type($type) {
  $is_distributable = variable_get('distribute_content_type_' . $type, 0);
  if ($is_distributable !== 0) {
    return TRUE;	
  }
  else {
  	return FALSE;
  }
}

/**
 * Determine whether all content types are set up 
 * for distribution.
 *
 * @return
 *   Array. Array of content types with a flag if
 *   their nodes are distributable or not.
 */
function distributor_service_get_all_types() {
  $content_types = node_get_types();
  foreach ($content_types as $type) {
  	$is_distributable = variable_get('distribute_content_type_' . $type->type,0);
  	if ($is_distributable !== 0) {
  		$types[$type->type] = $type->type;
  	}
  }
  return $types;  
}


/**
 * Looks up the node id on the local server based on the node id which
 * is being deleted on the other site and deletes the corresponding node.
 * 
 * @param $nid
 * the node id to look up
 */
function distributor_service_delete_node($nid) {
  // find the local node id.
  $result = db_query("SELECT nid FROM {content_retriever} WHERE original_nid = %d", $nid);
  $node_id = (int)db_result($result);
  
  //delete the node
  if ($node_id) {
    node_delete($node_id);
  }
  return $node_id;
}

/**
 * Services callback: return a view object.
 *
 * @param $view_name
 *   String. The view name.
 * @return
 *   Object. The view object.
 */
function distributor_service_get_view_object($view_name) {
  $view = views_get_view($view_name);
  
  return $view;
}

/**
 * Services callback: return CCK fields data from content_fields().
 *
 * @param $field_name
 *   String (optional). The field name.
 * @param $content_type_name
 *   String (optional). The content type name.
 * @return
 *   Array. The CCK data.
 */
function distributor_service_content_fields($field_name = NULL, $content_type_name = NULL) {
  // We don't depend on CCK, so check it is enabled.
  if (module_exists('content')) {
    $data = content_fields($field_name, $content_type_name);
    
    return $data;
  }
  else {
    return array();
  }
}

